@@ -44,6 +44,9 @@ namespace Beagle {
 		// A URI we can use to locate the source of this match.
 		private Uri uri = null;
 
+		// A URI of this Hit's container element
+		private Uri parent_uri = null;
+
 		// File, Web, MailMessage, IMLog, etc.
 		private string type = null;
 
@@ -81,7 +84,8 @@ namespace Beagle {
 		private DirectoryInfo directoryInfo = null;
 
 		//////////////////////////
-		
+
+		[XmlAttribute]
 		public int Id {
 			get { return id; }
 			set { id = value; }
@@ -93,7 +97,7 @@ namespace Beagle {
 			set { uri = value; }
 		}
 
-		[XmlElement ("Uri")]
+		[XmlAttribute ("Uri")]
 		public string UriAsString {
 			get {
 				return UriFu.UriToSerializableString (uri);
@@ -104,16 +108,42 @@ namespace Beagle {
 			}
 		}
 
+		[XmlIgnore]
+		public Uri ParentUri {
+			get { return parent_uri; }
+			set { parent_uri = value; }
+		}
+
+		[XmlAttribute ("ParentUri")]
+		public string ParentUriAsString {
+			get {
+				if (parent_uri == null)
+					return null;
+
+				return UriFu.UriToSerializableString (parent_uri);
+			}
+
+			set {
+				if (value == null)
+					parent_uri = null;
+				else
+					parent_uri = UriFu.UriStringToUri (value);
+			}
+		}
+
+		[XmlAttribute]
 		public string Type {
 			get { return type; }
 			set { type = value; }
 		}
 
+		[XmlAttribute]
 		public string MimeType {
 			get { return mimeType; }
 			set { mimeType = value; }
 		}
 	
+		[XmlAttribute]
 		public string Source {
 			get { return source; }
 			set { source = value; }
@@ -125,20 +155,24 @@ namespace Beagle {
 			set { sourceObject = value; }
 		}
 
+		[XmlAttribute]
 		public string SourceObjectName {
 			get { return source_object_name; }
 			set { source_object_name = value; }
 		}
 
+		[XmlAttribute]
 		public double Score {
 			get { return scoreRaw * scoreMultiplier; }
 		}
 
+		[XmlAttribute]
 		public double ScoreRaw {
 			get { return scoreRaw; }
 			set { scoreRaw = value; }
 		}
 
+		[XmlAttribute]
 		public double ScoreMultiplier {
 			get { return scoreMultiplier; }
 			set { 
@@ -330,6 +364,22 @@ namespace Beagle {
 			}
 		}
 
+		public string[] GetProperties (string key)
+		{
+			int first, top;
+			if (! FindProperty (key, out first, out top))
+				return null;
+
+			string[] values = new string [top - first];
+
+			for (int i = 0; first + i < top; i++) {
+				Property prop = properties [first + i] as Property;
+				values [i] = prop.Value;
+			}
+
+			return values;
+		}			
+
 		//////////////////////////
 
 		public override int GetHashCode ()
