@@ -37,14 +37,15 @@ namespace Beagle {
 
 	public enum QueryDomain {
 		Local        = 1,
-		Neighborhood = 2,
-		Global       = 4
+		System       = 2,
+		Neighborhood = 4,
+		Global       = 8
 	}
 
 	public class Query : RequestMessage {
 
 		// FIXME: This is a good default when on an airplane.
-		private Beagle.QueryDomain domainFlags = Beagle.QueryDomain.Local; 
+		private Beagle.QueryDomain domainFlags = QueryDomain.Local | QueryDomain.System; 
 
 		private ArrayList parts = new ArrayList ();
 		private ArrayList mimeTypes = new ArrayList ();
@@ -113,12 +114,11 @@ namespace Beagle {
 				this.CancelledEvent (response);
 		}
 
-		// FIXME: Useful?
 		private void OnError (ResponseMessage r)
 		{
 			ErrorResponse response = (ErrorResponse) r;
 
-			throw new Exception (response.Message);
+			throw new ResponseMessageException (response);
 		}
 
 		///////////////////////////////////////////////////////////////
