@@ -152,12 +152,11 @@ namespace Best {
 
 		int posX = 0, posY = 0;
 
-		public new void Show ()
+		public new void Present ()
 		{
-			base.Show ();	
+			base.Present ();	
 			
 			Move (posX, posY);
-			Present ();
 		}
 
 		public new void Hide ()
@@ -425,15 +424,16 @@ namespace Best {
 				string tmp = delayedQuery;
 				delayedQuery = null;
 				System.Console.WriteLine ("Delayed query fired");
-				Search (tmp);
+				StartQuery ();
 			}
 
 			return false;
 		}
 
-		private void QueueDelayedQuery ()
+		private void QueueDelayedQuery (string query)
 		{
-			GLib.Timeout.Add (1000, new GLib.TimeoutHandler (RunDelayedQuery));
+			delayedQuery = query;
+			GLib.Timeout.Add (10000, new GLib.TimeoutHandler (RunDelayedQuery));
 		}
 		
 		private void DoSearch (object o, EventArgs args)
@@ -447,13 +447,14 @@ namespace Best {
 		private void ChangeType (object o, EventArgs args)
 		{
 			Gtk.OptionMenu opt = (Gtk.OptionMenu) o;
-
 			TypeMenuItem mi = (TypeMenuItem) ((ArrayList) opt.Data["items"])[opt.History];
 
+			if (this.hit_type == mi.Type)
+				return;
+
 			this.hit_type = mi.Type;
 
 			root.SetSource (this.hit_type);
-
 			root.HitCollection.PageFirst ();
 			UpdatePage ();
 
@@ -509,51 +510,50 @@ namespace Best {
 			}
 		}
 
-		private void Search (String searchString)
+		private void StartQuery ()
 		{
 			try {
-				StoreSearch (searchString);
-				entry.GtkEntry.Text = searchString;
-
-				if (query != null) {
-					try { DetachQuery (); } catch (ObjectDisposedException e) {}
-				}
-
-				query = new Query ();
-				query.AddDomain (QueryDomain.Neighborhood);
-
-				// FIXME: Disable non-local searching for now.
-				//query.AddDomain (QueryDomain.Global);
-
-				query.AddText (searchString);
-				root.SetSource (hit_type);
-				
-				AttachQuery ();
-				
-				root.Query = query;
-				root.Start ();
-				
-				SetBusy (true);
 				query.SendAsync ();
+				SetBusy (true);
+			} catch (Beagle.ResponseMessageException e) {
+				QueueDelayedQuery (entry.GtkEntry.Text);
+				
+				/* To translators: {0} represents the current query keywords */
+				root.Error (String.Format (Catalog.GetString ("The query for <i>{0}</i> failed." +
+						    "<br>The likely cause is that the beagle daemon isn't running."), entry.GtkEntry.Text));
+				root.OfferDaemonRestart = true;
 			} catch (Exception e) {
-				delayedQuery = entry.GtkEntry.Text;
-				//DBusisms.BeagleUpAgain += QueueDelayedQuery;
-
-				if (e.ToString ().IndexOf ("com.novell.Beagle") != -1) {
-					/* To translators: {0} represents the current query keywords */
-					root.Error (String.Format (Catalog.GetString ("The query for <i>{0}</i> failed." +
-						    "<br>The likely cause is that the beagle daemon isn't running."),searchString));
-					root.OfferDaemonRestart = true;
-				} else if (e.ToString().IndexOf ("Unable to determine the address") != -1) {
-					/* To translators: {0} represents the current query keywords */
-					root.Error (String.Format (Catalog.GetString ("The query for <i>{0}</i> failed.<br>" +
-						    "The session bus isn't running.  See http://beaglewiki.org/index.php/Installing%20Beagle for information on setting up a session bus."),searchString));
-				} else {
-					/* To translators: {0} represents the current query keywords, {1} contains the errormessage */
-					root.Error (String.Format (Catalog.GetString ("The query for <i>{0}</i> failed with error:<br>{1}<br>"),searchString, e));
-				}
+				/* To translators: {0} represents the current query keywords, {1} contains the errormessage */
+				root.Error (String.Format (Catalog.GetString ("The query for <i>{0}</i> failed with error:<br>{1}<br>"), 
+							   entry.GtkEntry.Text, e));
 			}
+		}
 
+		private void Search (String searchString)
+		{
+			StoreSearch (searchString);
+			entry.GtkEntry.Text = searchString;
+
+			if (query != null) {
+				try { DetachQuery (); } catch (ObjectDisposedException e) {}
+			}
+			
+			query = new Query ();
+			query.AddDomain (QueryDomain.Neighborhood);
+			
+			// FIXME: Disable non-local searching for now.
+			//query.AddDomain (QueryDomain.Global);
+			
+			query.AddText (searchString);
+			root.SetSource (hit_type);
+			
+			AttachQuery ();
+			
+			root.Query = query;
+			root.Start ();
+			
+			StartQuery ();
+			
 			UpdatePage ();
 		}
 
@@ -582,6 +582,12 @@ namespace Best {
 		{
 			return recentSearches;
 		}
+
+		public void ClearHistory ()
+		{
+			entry.ClearHistory ();
+			recentSearches.Clear ();
+		}
 		
 		public void QuickSearch (string query) 
 		{
