@@ -88,14 +88,16 @@ namespace Beagle.Tile {
 		public int Weight {
 			get {
 				int weight = 0;
-				if (Uri != null)
-					++weight;
-				if (Type != null)
-					++weight;
-				if (MimeType != null)
-					++weight;
+
 				if (Source != null)
-					++weight;
+					weight += 8;				
+				if (Type != null)
+					weight += 4;
+				if (MimeType != null)
+					weight += 2;
+				if (Uri != null)
+					weight += 1;
+
 				return weight;
 			}
 		}
