@@ -1,4 +1,3 @@
-
 //
 // EvolutionMailDriver.cs
 //
@@ -48,7 +47,14 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 		private MailCrawler crawler;
 
-		public EvolutionMailQueryable () : base ("MailIndex")
+		// Index versions
+		// 1: Original version, stored all recipient addresses as a
+		//    RFC822 string
+		// 2: Stores recipients in separate properties,
+		//    filters/indexes all attachments
+		private const int INDEX_VERSION = 2;
+
+		public EvolutionMailQueryable () : base ("MailIndex", INDEX_VERSION)
 		{
 			this.local_path = Path.Combine (PathFinder.HomeDir, ".evolution/mail/local");
 			this.imap_path = Path.Combine (PathFinder.HomeDir, ".evolution/mail/imap");
@@ -298,15 +304,30 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 		public void IndexSummary (FileInfo summaryInfo)
 		{
+			// If there's already a task running for this folder,
+			// don't interrupt it.
+			if (ThisScheduler.ContainsByTag (summaryInfo.FullName)) {
+				Logger.Log.Debug ("Not adding task for already running task: {0}", summaryInfo.FullName);
+				return;
+			}
+
 			EvolutionMailIndexableGeneratorImap generator = new EvolutionMailIndexableGeneratorImap (this, summaryInfo);
 			Scheduler.Task task;
 			task = NewAddTask (generator, new Scheduler.Hook (generator.Checkpoint));
+			task.Tag = summaryInfo.FullName;
 			// IndexableGenerator tasks default to having priority Scheduler.Priority Generator
 			ThisScheduler.Add (task);
 		}
 
 		public void IndexMbox (FileInfo mboxInfo)
 		{
+			// If there's already a task running for this mbox,
+			// don't interrupt it.
+			if (ThisScheduler.ContainsByTag (mboxInfo.FullName)) {
+				Logger.Log.Debug ("Not adding task for already running task: {0}", mboxInfo.FullName);
+				return;
+			}
+
 			EvolutionMailIndexableGeneratorMbox generator = new EvolutionMailIndexableGeneratorMbox (this, mboxInfo);
 			Scheduler.Task task;
 			task = NewAddTask (generator, new Scheduler.Hook (generator.Checkpoint));
