@@ -44,7 +44,15 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 		private static bool gmime_initialized = false;
 
-		private EvolutionMailQueryable queryable;
+		private static ArrayList excludes = new ArrayList ();
+
+		static EvolutionMailIndexableGenerator () {
+			foreach (ExcludeItem exclude in Conf.Indexing.Excludes)
+				if (exclude.Type == ExcludeType.MailFolder)
+					excludes.Add (exclude);
+		}
+
+		protected EvolutionMailQueryable queryable;
 
 		protected string account_name, folder_name;
 		protected int count, indexed_count;
@@ -81,149 +89,20 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			else
 				return false;
 		}
-					
-		protected bool CrawlNeeded ()
+
+		protected bool IgnoreFolder (string path)
 		{
-			string timeStr;
-
-			try {
-				timeStr = ExtendedAttribute.Get (this.CrawlFile.FullName, "LastCrawl");
-			} catch {
-				EvolutionMailQueryable.log.Debug ("Unable to get last crawl time on {0}",
-								  this.CrawlFile.FullName);
-				return true;
+			// FIXME: Use System.IO.Path
+			foreach (ExcludeItem exclude in excludes) {
+				if (exclude.IsMatch (path))
+					return true;
 			}
-
-			DateTime lastCrawl = StringFu.StringToDateTime (timeStr);
-
-			if (this.CrawlFile.LastWriteTime > lastCrawl)
-				return true;
-			else
-				return false;
+			return false;
 		}
-
+					
 		protected void CrawlFinished ()
 		{
-			try {
-				ExtendedAttribute.Set (this.CrawlFile.FullName, "LastCrawl",
-						       StringFu.DateTimeToString (DateTime.Now));
-			} catch {
-				EvolutionMailQueryable.log.Debug ("Unable to set last crawl time on {0}",
-								  this.CrawlFile.FullName);
-			}
-		}
-
-		static public bool SupportedContentType (GMime.ContentType type)
-		{
-			return type.Type == "text" || type.Type == "Text" || type.Type == "TEXT";
-		}
-
-		protected class PartHandler {
-			private MultiReader reader = null;
-			private int depth = 0;
-
-			public void OnEachPart (GMime.Object part)
-			{
-				if (this.reader == null)
-					this.reader = new MultiReader ();
-
-				//for (int i = 0; i < depth; i++)
-				//  Console.Write ("  ");
-				//Console.WriteLine ("Content-Type: {0}", part.ContentType);
-
-				++depth;
-
-				if (part is GMime.MessagePart) {
-					GMime.MessagePart msg_part = (GMime.MessagePart) part;
-
-					using (GMime.Message message = msg_part.Message) {
-						using (GMime.Object subpart = message.MimePart)
-							this.OnEachPart (subpart);
-					}
-				} else if (part is GMime.Multipart) {
-					GMime.Multipart multipart = (GMime.Multipart) part;
-
-					int num_parts = multipart.Number;
-					for (int i = 0; i < num_parts; i++) {
-						using (GMime.Object subpart = multipart.GetPart (i))
-							this.OnEachPart (subpart);
-					}
-				} else if (SupportedContentType (part.ContentType)) {
-					MemoryStream stream = new MemoryStream (part.GetData ());
-					StreamReader reader = new StreamReader (stream);
-
-					// If this is the first part, we need to skip past the
-					// message headers --- we want to store that data in
-					// properties, not index it as text.
-					if (this.reader.Count == 0) {
-						string line;
-						do {
-							line = reader.ReadLine ();
-						} while (line != null && line != "");
-					}
-
-					this.reader.Add (reader);
-				} 
-
-				--depth;
-			}
-
-			public static MultiReader GetReader (GMime.Message message)
-			{
-				// FIXME: This will work for now, but really we need
-				// to make it so that we can index multiple things
-				// from a single indexable.
-				PartHandler handler = new PartHandler ();
-				//message.ForeachPart (new GMime.PartFunc (handler.OnEachPart));
-				using (GMime.Object mime_part = message.MimePart)
-					handler.OnEachPart (mime_part);
-				return handler.reader;
-			}
-
-		}
-
-		protected static Stream ReadAppData (string name)
-		{
-			string path = Path.Combine (Path.Combine (PathFinder.StorageDir, "MailIndex"), name);
-			return new FileStream (path, System.IO.FileMode.Open, FileAccess.Read, FileShare.Read);
-		}
-
-		protected static string ReadAppDataLine (string name)
-		{
-			Stream stream;
-
-			try {
-				stream = ReadAppData (name);
-			} catch (FileNotFoundException) {
-				return null;
-			}
-
-			StreamReader sr = new StreamReader (stream);
-			string line = sr.ReadLine ();
-			sr.Close ();
-			return line;
-		}
-
-		protected static Stream WriteAppData (string name)
-		{
-			string path = Path.Combine (Path.Combine (PathFinder.StorageDir, "MailIndex"), name);
-			return new FileStream (path, System.IO.FileMode.Create, FileAccess.Write, FileShare.None);
-		}
-
-		protected static void WriteAppDataLine (string name, string line)
-		{
-			if (line == null) {
-				string path = Path.Combine (Path.Combine (PathFinder.StorageDir, "MailIndex"), name);
-
-				if (File.Exists (path))
-					File.Delete (path);
-
-				return;
-			}
-
-			StreamWriter sw = new StreamWriter (WriteAppData (name));
-			sw.WriteLine (line);
-			sw.Close ();
+			this.queryable.FileAttributesStore.AttachTimestamp (this.CrawlFile.FullName, DateTime.Now);
 		}
 
 		public string StatusName {
@@ -250,49 +129,6 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		{
 			return this.CrawlFile.FullName.GetHashCode ();
 		}
-
-		// FIXME FIXME FIXME!
-		// This is a fragile piece-of-shit parser.
-		// This will not do the right thing if the address is quoted
-		// or contains special characters.  For example, we will do the wrong thing
-		// with:
-		// "Foo, bar" <foo@bar.com>, Zoo <zoo@bar.com>"
-		// Also, this almost certainly won't deal well w/ broken addresses.
-		static protected ICollection ExtractAddresses (string addr_str)
-		{
-			string [] split = addr_str.Split (',');
-
-			ArrayList addresses = new ArrayList ();
-			foreach (string part in split) {
-				if (part == "")
-					continue;
-				if (part [part.Length - 1] == '>') {
-					int i = part.LastIndexOf ('<');
-					addresses.Add (part.Substring (i+1, part.Length-i-2));
-				} else if (part [part.Length - 1] == ')') {
-					int i = part.LastIndexOf ('(');
-					addresses.Add (part.Substring (i+1, part.Length-i-2));
-				} else {
-					addresses.Add (part);
-				}
-			}
-			
-			return addresses;
-		}
-
-		static protected void MapAddressStringToProperties (Indexable indexable,
-								    string    property_key,
-								    string    address_string)
-		{
-			if (address_string == null || address_string.Length == 0)
-				return;
-
-			foreach (string one_addr in ExtractAddresses (address_string)) {
-				Property prop;
-				prop = Property.NewKeyword (property_key, one_addr);
-				indexable.AddProperty (prop);
-			}
-		}
 	}
 
 	public class EvolutionMailIndexableGeneratorMbox : EvolutionMailIndexableGenerator {
@@ -333,13 +169,16 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 			if (this.IsSpamFolder (this.folder_name))
 				return false;
-			
+
+			if (this.IgnoreFolder (this.mbox_info.FullName))
+				return false;
+
 			return true;
 		}
 
 		private long MboxLastOffset {
 			get {
-				string offset_str = ReadAppDataLine ("offset-" + this.folder_name.Replace ('/', '-'));
+				string offset_str = this.queryable.ReadDataLine ("offset-" + this.folder_name.Replace ('/', '-'));
 				long offset = Convert.ToInt64 (offset_str);
 
 				Logger.Log.Debug ("mbox {0} offset is {1}", this.mbox_info.Name, offset);
@@ -347,7 +186,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			}
 
 			set {
-				WriteAppDataLine ("offset-" + this.folder_name.Replace ('/', '-'), value.ToString ());
+				this.queryable.WriteDataLine ("offset-" + this.folder_name.Replace ('/', '-'), value.ToString ());
 			}
 		}
 
@@ -428,7 +267,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				string uid_str = x_evolution.Substring (0, separator_idx);
 				string uid = Convert.ToUInt32 (uid_str, 16).ToString (); // ugh.
 				uint flags = Convert.ToUInt32 (x_evolution.Substring (separator_idx + 1), 16);
-				
+
 				Indexable indexable = this.GMimeMessageToIndexable (uid, message, flags);
 				
 				if (indexable == null)
@@ -459,44 +298,39 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 			indexable.Timestamp = message.Date;
 			indexable.Type = "MailMessage";
-			indexable.MimeType = "text/plain";
+			indexable.MimeType = "message/rfc822";
 			indexable.CacheContent = false;
 
 			indexable.AddProperty (Property.NewKeyword ("fixme:client", "evolution"));
-
-			string subject = GMime.Utils.HeaderDecodePhrase (message.Subject);
-			indexable.AddProperty (Property.New ("dc:title", subject));
-			indexable.AddProperty (Property.New ("fixme:subject", subject));
-
 			indexable.AddProperty (Property.NewKeyword ("fixme:account", "Local"));
                         indexable.AddProperty (Property.NewKeyword ("fixme:folder", this.folder_name));
 
-			string to_str = message.GetRecipientsAsString (GMime.Message.RecipientType.To);
-			string cc_str = message.GetRecipientsAsString (GMime.Message.RecipientType.Cc);
-			string from_str = GMime.Utils.HeaderDecodePhrase (message.Sender);
-                        indexable.AddProperty (Property.NewKeyword ("fixme:to", to_str));       
-			indexable.AddProperty (Property.NewKeyword ("fixme:cc", cc_str));
-			indexable.AddProperty (Property.NewKeyword ("fixme:from", from_str));
+			GMime.InternetAddressList addrs;
 
-			if (this.folder_name == "Sent") {
+			addrs = message.GetRecipients (GMime.Message.RecipientType.To);
+			foreach (GMime.InternetAddress ia in addrs) {
+				if (this.folder_name == "Sent")
+					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+			}
+			addrs.Dispose ();
+
+			addrs = message.GetRecipients (GMime.Message.RecipientType.Cc);
+			foreach (GMime.InternetAddress ia in addrs) {
+				if (this.folder_name == "Sent")
+					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+			}
+			addrs.Dispose ();
+
+			addrs = GMime.InternetAddressList.ParseString (GMime.Utils.HeaderDecodePhrase (message.Sender));
+			foreach (GMime.InternetAddress ia in addrs) {
+				if (this.folder_name != "Sent")
+					indexable.AddProperty (Property.NewKeyword ("fixme:gotFrom", ia.Addr));
+			}
+			addrs.Dispose ();
+
+			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewFlag ("fixme:isSent"));
 
-				MapAddressStringToProperties (indexable, "fixme:sentTo", to_str);
-				MapAddressStringToProperties (indexable, "fixme:sentTo", cc_str);
-			} else {
-				MapAddressStringToProperties (indexable, "fixme:gotFrom", from_str);
-			}
-
-			if (message.MimePart is GMime.Multipart || message.MimePart is GMime.MessagePart)
-				indexable.AddProperty (Property.NewFlag ("fixme:hasAttachments"));
-
-			string list_id = message.GetHeader ("List-Id");
-
-			if (list_id != null) {
-				// FIXME: Might need some additional parsing.
-				indexable.AddProperty (Property.NewKeyword ("fixme:mlist", GMime.Utils.HeaderDecodePhrase (list_id)));
-			}
-
 			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewDate ("fixme:sentdate", message.Date));
 			else
@@ -521,8 +355,11 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 			if (CheckFlags (flags, Camel.CamelFlags.AnsweredAll))
 				indexable.AddProperty (Property.NewFlag ("fixme:isAnsweredAll"));
-			
-			indexable.SetTextReader (PartHandler.GetReader (message));
+
+			MemoryStream stream = new MemoryStream ();
+			message.WriteToStream (stream);
+			stream.Seek (0, SeekOrigin.Begin);
+			indexable.SetBinaryStream (stream);
 
 			return indexable;
 		}
@@ -653,7 +490,6 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				if (semicolon != -1)
 					imap_url = imap_url.Substring (0, semicolon) + imap_url.Substring (user_end);
 
-
 				// Escape out additional @s in the name.  I hate the class libs so much.
 				int lastIdx = this.imap_name.LastIndexOf ('@');
 				if (this.imap_name.IndexOf ('@') != lastIdx) {
@@ -683,6 +519,10 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			DirectoryInfo dir_info = new DirectoryInfo (dir_name);
 			if (this.IsSpamFolder (dir_info.Name))
 				return false;
+
+			// Check if the folder is listed in the configuration as to be excluded from indexing
+			if (this.IgnoreFolder (dir_info.FullName))
+				return false;
 					
 			this.folder_name = GetFolderName (new DirectoryInfo (dir_name));
 
@@ -704,7 +544,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			BinaryFormatter formatter;
 
 			try {
-				cacheStream = ReadAppData (this.FolderCacheName);
+				cacheStream = this.queryable.ReadDataStream (this.FolderCacheName);
 				formatter = new BinaryFormatter ();
 				this.mapping = formatter.Deserialize (cacheStream) as Hashtable;
 				cacheStream.Close ();
@@ -723,7 +563,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			Stream cacheStream;
 			BinaryFormatter formatter;
 			
-			cacheStream = WriteAppData (this.FolderCacheName);
+			cacheStream = this.queryable.WriteDataStream (this.FolderCacheName);
 			formatter = new BinaryFormatter ();
 			formatter.Serialize (cacheStream, mapping);
 			cacheStream.Close ();
@@ -740,9 +580,11 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				bool cache_loaded = this.LoadCache ();
 
 				this.deleted_list = new ArrayList (this.mapping.Keys);
+				this.deleted_list.Sort ();
+				Logger.Log.Debug ("Deleted list starting at {0} for {1}", this.deleted_list.Count, this.folder_name);
 
 				// Check to see if we even need to bother walking the summary
-				if (cache_loaded && ! CrawlNeeded ()) {
+				if (cache_loaded && this.queryable.FileAttributesStore.IsUpToDate (this.CrawlFile.FullName)) {
 					EvolutionMailQueryable.log.Debug ("{0}: summary has not been updated; crawl unncessary", this.folder_name);
 					return false;
 				}
@@ -767,6 +609,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			if (this.summary_enumerator.MoveNext ())
 				return true;
 
+			Logger.Log.Debug ("Queuing up {0} removals for deleted messages in ", this.deleted_list.Count, this.folder_name);
 			foreach (string uid in this.deleted_list) {
 				Uri uri = EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, uid);
 
@@ -788,32 +631,6 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			return false;
 		}
 
-		private MultiReader GetMessageData (string file)
-		{
-			try {
-				InitializeGMime ();
-			} catch (Exception e) {
-				Logger.Log.Warn ("Caught exception trying to initialize gmime:");
-				Logger.Log.Warn (e);
-				return null;
-			}
-
-			int fd = Syscall.open (file, OpenFlags.O_RDONLY);
-			GMime.StreamFs stream = new GMime.StreamFs (fd);
-			GMime.Parser parser = new GMime.Parser (stream);
-
-			MultiReader reader = null;
-
-			using (GMime.Message message = parser.ConstructMessage ())
-				reader = PartHandler.GetReader (message);
-
-			stream.Close ();
-			stream.Dispose ();
-			parser.Dispose ();
-
-			return reader;
-		}
-
 		// Kind of nasty, but we need the function.
 		[System.Runtime.InteropServices.DllImport("libglib-2.0.so.0")]
 		static extern int g_str_hash (string str);
@@ -830,8 +647,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			++this.count;
 
 			// Try to load the cached message data off disk
-			if (this.mapping[mi.uid] == null || (uint) mapping[mi.uid] != mi.flags) {
-				TextReader msgReader = null;
+			object flags = this.mapping[mi.uid];
+
+			if (flags == null || (uint) flags != mi.flags) {
 				string msg_file;
 
 				if (this.backend_type == ImapBackendType.Imap)
@@ -843,27 +661,21 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 					string cache_path = String.Format ("cache/{0:x}/{1}", hash, mi.uid);
 					msg_file = Path.Combine (summary_info.DirectoryName, cache_path);
 				}
-				
-				// FIXME - Filters really need to be rearchitected so that we can
-				// pass multiple attachments of different types into the indexable.
-				if (File.Exists (msg_file))
-					msgReader = GetMessageData (msg_file);
 
-				indexable = this.CamelMessageToIndexable (mi, msgReader);
+				indexable = this.CamelMessageToIndexable (mi, msg_file);
 
 				this.mapping[mi.uid] = mi.flags;
 
-				if (indexable != null)
+				if (indexable != null) {
 					++this.indexed_count;
-			} 
-
-			if (indexable != null) {
+					this.deleted_list.Remove (mi.uid);
+					
+					// HACK: update your recipients
+					EvolutionMailQueryable.AddAsYourRecipient (indexable);
+				}
+			} else if (flags != null)
 				this.deleted_list.Remove (mi.uid);
 
-				// HACK: update your recipients
-				EvolutionMailQueryable.AddAsYourRecipient (indexable);
-			}
-
 			return indexable;
 		}
 
@@ -872,26 +684,70 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			return EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, message_info.uid);
 		}
 
-		private Indexable CamelMessageToIndexable (Camel.MessageInfo messageInfo, TextReader msgReader)
+		private Indexable CamelMessageToIndexable (Camel.MessageInfo messageInfo, string msg_file)
 		{
 			// Don't index messages flagged as junk
 			if (messageInfo.IsJunk)
 				return null;
 
+			// Many properties will be set by the filter when
+			// processing the cached data, if it's there.  So
+			// don't set a number of properties in that case.
+			bool have_content = File.Exists (msg_file);
+
 			Uri uri = CamelMessageUri (messageInfo);
 			Indexable indexable = new Indexable (uri);
 
 			indexable.Timestamp = messageInfo.Date;
+			indexable.MimeType = "message/rfc822";
 			indexable.Type = "MailMessage";
 
-			indexable.AddProperty (Property.New ("dc:title", messageInfo.subject));
-			indexable.AddProperty (Property.New ("fixme:subject",  messageInfo.subject));
-
 			indexable.AddProperty (Property.NewKeyword ("fixme:account",  this.imap_name));
                         indexable.AddProperty (Property.NewKeyword ("fixme:folder",   this.folder_name));
-                        indexable.AddProperty (Property.NewKeyword ("fixme:to",       messageInfo.to));
-			indexable.AddProperty (Property.NewKeyword ("fixme:from",     messageInfo.from));
-                        indexable.AddProperty (Property.NewKeyword ("fixme:cc",       messageInfo.cc));
+			
+			if (!have_content)
+				indexable.AddProperty (Property.New ("dc:title", messageInfo.subject));
+
+			GMime.InternetAddressList addrs;
+			addrs = GMime.InternetAddressList.ParseString (messageInfo.to);
+			foreach (GMime.InternetAddress ia in addrs) {
+				if (!have_content) {
+					indexable.AddProperty (Property.NewKeyword ("fixme:to", ia.ToString (false)));
+					indexable.AddProperty (Property.NewKeyword ("fixme:to_address", ia.Addr));
+					indexable.AddProperty (Property.NewKeyword ("fixme:to_name", ia.Name));
+				}
+
+				if (this.folder_name == "Sent")
+					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+			}
+			addrs.Dispose ();
+
+			addrs = GMime.InternetAddressList.ParseString (messageInfo.cc);
+			foreach (GMime.InternetAddress ia in addrs) {
+				if (!have_content) {
+					indexable.AddProperty (Property.NewKeyword ("fixme:cc", ia.ToString (false)));
+					indexable.AddProperty (Property.NewKeyword ("fixme:cc_address", ia.Addr));
+					indexable.AddProperty (Property.NewKeyword ("fixme:cc_name", ia.Name));
+				}
+
+				if (this.folder_name == "Sent")
+					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+			}
+			addrs.Dispose ();
+
+			addrs = GMime.InternetAddressList.ParseString (messageInfo.from);
+			foreach (GMime.InternetAddress ia in addrs) {
+				if (!have_content) {
+					indexable.AddProperty (Property.NewKeyword ("fixme:from", ia.ToString (false)));
+					indexable.AddProperty (Property.NewKeyword ("fixme:from_address", ia.Addr));
+					indexable.AddProperty (Property.NewKeyword ("fixme:from_name", ia.Name));
+				}
+
+				if (this.folder_name != "Sent")
+					indexable.AddProperty (Property.NewKeyword ("fixme:gotFrom", ia.Addr));
+			}
+			addrs.Dispose ();
+
                         indexable.AddProperty (Property.NewKeyword ("fixme:mlist",    messageInfo.mlist));
                         indexable.AddProperty (Property.NewKeyword ("fixme:flags",    messageInfo.flags));
 
@@ -901,15 +757,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			if (messageInfo.sent != DateTime.MinValue)
 				indexable.AddProperty (Property.NewDate ("fixme:sentdate", messageInfo.sent));
 
-			if (this.folder_name == "Sent") {
+			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewFlag ("fixme:isSent"));
 
-				MapAddressStringToProperties (indexable, "fixme:sentTo", messageInfo.to);
-				MapAddressStringToProperties (indexable, "fixme:sentTo", messageInfo.cc);
-			} else {
-				MapAddressStringToProperties (indexable, "fixme:gotFrom", messageInfo.from);
-			}
-
 			if (messageInfo.IsAnswered)
 				indexable.AddProperty (Property.NewFlag ("fixme:isAnswered"));
 
@@ -925,14 +775,14 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			if (messageInfo.IsSeen)
 				indexable.AddProperty (Property.NewFlag ("fixme:isSeen"));
 
-			if (messageInfo.HasAttachments)
+			if (messageInfo.HasAttachments && !have_content)
 				indexable.AddProperty (Property.NewFlag ("fixme:hasAttachments"));
 
 			if (messageInfo.IsAnsweredAll)
 				indexable.AddProperty (Property.NewFlag ("fixme:isAnsweredAll"));
 
-			if (msgReader != null)
-				indexable.SetTextReader (msgReader);
+			if (have_content)
+				indexable.ContentUri = UriFu.PathToFileUri (msg_file);
 			else
 				indexable.NoContent = true;
 
