@@ -409,12 +409,11 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 
 		object big_lock = new object ();
-		string fingerprint;
 		ArrayList roots = new ArrayList ();
 		Hashtable path_cache = new Hashtable ();
 		Hashtable by_unique_id = new Hashtable ();
 		Queue to_be_scanned = new Queue ();
-		FileNameFilter filter = new FileNameFilter ();
+		FileNameFilter filter;
 
 		IFileEventBackend event_backend;
 		int needs_crawl_count = 0;
@@ -430,10 +429,10 @@ namespace Beagle.Daemon.FileSystemQueryable {
 					IFileEventBackend event_backend)
 		{
 			this.event_backend = event_backend;
-			this.fingerprint = index_fingerprint;
 
 			unique_id_store = new UniqueIdStore (index_directory, index_fingerprint);
 			name_index = new NameIndex (index_directory, index_fingerprint);
+			filter = new FileNameFilter (this);
 
 			IFileAttributesStore backing_store_i;
 			backing_store_i = new FileAttributesStore_Mixed (index_directory, index_fingerprint);
@@ -444,6 +443,35 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			fa_store = new FileAttributesStore (this);
 		}
 
+		public void LoadConfiguration () 
+		{
+			if (Conf.Indexing.IndexHomeDir)
+				AddRoot (PathFinder.HomeDir);
+			
+			foreach (string root in Conf.Indexing.Roots)
+				AddRoot (root);
+
+			Conf.Subscribe (typeof (Conf.IndexingConfig), OnConfigurationChanged);
+		}
+
+		private void OnConfigurationChanged (Conf.Section section)
+		{
+			ArrayList roots_wanted = new ArrayList (Conf.Indexing.Roots);
+			
+			if (Conf.Indexing.IndexHomeDir)
+				roots_wanted.Add (PathFinder.HomeDir);
+			
+			IList roots_to_add, roots_to_remove;
+
+			ArrayFu.IntersectListChanges (roots_wanted, RootsAsPaths, out roots_to_add, out roots_to_remove);
+
+			foreach (string root in roots_to_remove)
+				RemoveRoot (root);
+
+			foreach (string root in roots_to_add)
+				AddRoot (root);
+		}
+
 		// I'd rather not expose these, but we really can't avoid it.
 		public UniqueIdStore UniqueIdStore {
 			get { return unique_id_store; }
@@ -461,6 +489,18 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			get { lock (big_lock) return roots.Clone () as ArrayList; }
 		}
 
+		private IList RootsAsPaths {
+			get {
+				ArrayList roots = new ArrayList ();
+
+				foreach (Directory root in Roots) {
+					roots.Add (root.FullName);
+				}
+
+				return roots;
+			}
+		}
+
 		public Directory AddRoot (string path)
 		{
 			// Remove trailing directory separators, which cause Path.FileName
@@ -470,8 +510,6 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			while (path.Length > 0 && path [path.Length-1] == System.IO.Path.DirectorySeparatorChar)
 				path = path.Substring (0, path.Length-1);
 
-			Logger.Log.Debug ("Adding root {0}", path);
-				
 			DirectoryPrivate root = new DirectoryPrivate (big_lock);
 			root.InitRoot (System.IO.Path.GetDirectoryName (path), 
 				       System.IO.Path.GetFileName (path));
@@ -485,13 +523,11 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			lock (big_lock) {
 				// FIXME: We also should make sure the path is not a parent or child
 				// of any existing root.
-				foreach (Directory existing_root in roots) {
-					if (existing_root.FullName == root.FullName) {
-						throw new Exception ("Attempt to re-add root " + root.FullName);
-						// FIXME: Is there a case where we would
-						// want to just return existing_root?
-					}
-				}
+				foreach (Directory existing_root in roots)
+					if (existing_root.FullName == root.FullName)
+						return existing_root;
+
+				Logger.Log.Debug ("Adding root {0}", path);
 				roots.Add (root);
 				to_be_scanned.Enqueue (root);
 				if (to_be_scanned.Count == 1)
@@ -513,6 +549,28 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			return root;
 		}
 
+		public bool RemoveRoot (string path) 
+		{
+			return RemoveRoot (GetDirectoryByPath (path));
+		}
+
+		public bool RemoveRoot (Directory root) 
+		{
+			DirectoryPrivate priv = (DirectoryPrivate) root;
+			
+			lock (big_lock) {
+				if (!roots.Contains (root))
+					return false;
+
+				Logger.Log.Debug ("Removing root {0}", root.FullName);
+
+				roots.Remove (root);
+				RecursivelyRemove_Unlocked (priv);
+			}
+			
+			return true;
+		}
+
 		///////////////////////////////////////////////////////////////////////////
 
 		public Directory GetDirectoryByUniqueId (Guid uid)
@@ -548,6 +606,8 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 					int i = 0;
 
+					// FIXME: FH
+					
 					// Next, find the correct root
 					path = "";
 					for (i = 0; i < path_parts.Count && dir == null; ++i) {
@@ -574,6 +634,23 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			}
 		}
 
+		private void RecursivelyRemove_Unlocked (DirectoryPrivate priv)
+		{
+			if (priv.WatchHandle != null)
+				event_backend.ForgetWatch (priv.WatchHandle);				
+
+			if (priv.FullNameIsCached)
+				path_cache.Remove (priv.FullName);
+			
+			by_unique_id.Remove (priv.UniqueId);
+			unique_id_store.Drop (priv.UniqueId);
+
+			foreach (Directory subdir in priv.Children)
+				RecursivelyRemove_Unlocked ((DirectoryPrivate) subdir);
+
+			//priv.Detatch_Unlocked ();
+		}
+
 		private void RecursivelyRemoveFromPathCache_Unlocked (DirectoryPrivate priv)
 		{
 			if (priv.FullNameIsCached) {
@@ -694,6 +771,18 @@ namespace Beagle.Daemon.FileSystemQueryable {
 				}
 			}
 
+			// FIXME: This does not take in to account that we might have a better matching filter to use now
+			// That, however, is kind of expensive to figure out since we'd have to do mime-sniffing and shit.
+			if (attr.FilterName != null && attr.FilterVersion > 0) {
+				int current_filter_version = FilterFactory.GetFilterVersion (attr.FilterName);
+
+				if (current_filter_version > attr.FilterVersion) {
+					if (Debug)
+						Logger.Log.Debug ("*** Newer filter version found for filter {0}, re-indexing", attr.FilterName);
+					return RequiredAction.Index;
+				}
+			}
+
 			if (Debug)
 				Logger.Log.Debug ("*** Doing nothing to {0}", path);
 
@@ -738,10 +827,10 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 			System.IO.DirectoryInfo info = new System.IO.DirectoryInfo (priv.FullName);
 
-			// It's the call to info.GetDirectories() that may
+			// It's the call to GetDirectoryInfos() that may
 			// trigger the exception caught below.
 			try {
-				foreach (System.IO.DirectoryInfo subinfo in info.GetDirectories ()) {
+				foreach (System.IO.DirectoryInfo subinfo in DirectoryWalker.GetDirectoryInfos (info)) {
 					if (! Ignore (subinfo.FullName)) {
 						if (! priv.HasChildWithName (subinfo.Name))
 							AddChild_Unlocked (priv, subinfo.Name);
@@ -749,7 +838,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 					if (known_children != null)
 						known_children.Remove (subinfo.Name);
 				}
-			} catch (UnauthorizedAccessException e) {
+			} catch (System.IO.DirectoryNotFoundException e) {
 				Logger.Log.Warn ("Skipping over {0}: {1}", priv.FullName, e.Message);
 			}
 
@@ -788,6 +877,11 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			lock (big_lock) {
 				int old_needs_crawl_count = needs_crawl_count;
 				while (to_be_scanned.Count > 0) {
+					if (Shutdown.ShutdownRequested) {
+						Logger.Log.Debug ("Bailing out of subdir scan -- shutdown requested");
+						return;
+					}
+
 					Directory dir;
 					dir = to_be_scanned.Dequeue () as Directory;
 					ScanOne_Unlocked (dir);
@@ -926,6 +1020,26 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 		///////////////////////////////////////////////////////////////////////////
 
+		public void MarkAsFiltered (string path, string filter_name, int filter_version)
+		{
+			FileAttributes attr = backing_store.ReadOrCreate (path);
+
+			if (attr == null)
+				return;
+
+			attr.FilterName = filter_name;
+			attr.FilterVersion = filter_version;
+
+			try {
+				Write (attr);
+			} catch (Exception ex) {
+				Logger.Log.Error ("Error writing filtered status attributes: {0}", path);
+				Logger.Log.Error (ex);
+			}
+		}
+
+		///////////////////////////////////////////////////////////////////////////
+
 		private void AddChild_Unlocked (DirectoryPrivate parent, string child_name)
 		{
 			DirectoryPrivate child = new DirectoryPrivate (big_lock);
@@ -964,6 +1078,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 				RecursivelyRemoveFromPathCache_Unlocked (priv);
 				priv.Detatch_Unlocked ();
+
 			}
 		}
 
@@ -1024,9 +1139,9 @@ namespace Beagle.Daemon.FileSystemQueryable {
 		// Search by filename
 
 		// Returns a collection of internal Uris
-		public ICollection Search (QueryBody body, ICollection list_of_uris)
+		public ICollection Search (Query query, ICollection list_of_uris)
 		{
-			return name_index.Search (body, list_of_uris);
+			return name_index.Search (query, list_of_uris);
 		}
 
 		//////////////////////////////////////////////////////////////////////////////////
@@ -1045,6 +1160,13 @@ namespace Beagle.Daemon.FileSystemQueryable {
 				FileAttributes attr;
 				attr = backing_store.ReadOrCreate (path);
 				unique_id = attr.UniqueId;
+
+				if (! unique_id_store.IsCached (attr.UniqueId)) {
+					string dir_name = System.IO.Path.GetDirectoryName (attr.Path);
+					string file_name = System.IO.Path.GetFileName (attr.Path);
+					Directory dir = GetDirectoryByPath (dir_name);
+					unique_id_store.Add (attr.UniqueId, dir.UniqueId, file_name, false);
+				}
 			} else {
 				// Maybe the file got deleted.  If so, try to get it from
 				// the unique id store.
@@ -1077,8 +1199,13 @@ namespace Beagle.Daemon.FileSystemQueryable {
 		public bool InternalUriIsValid (Uri internal_uri)
 		{
 			string path = unique_id_store.GetPathByUidUri (internal_uri);
+
 			if (path == null)
 				return false;
+
+			if (Ignore (path))
+				return false;
+			
 			return System.IO.File.Exists (path) || System.IO.Directory.Exists (path);
 		}
 
