@@ -55,6 +55,10 @@ namespace Lucene.Net.Store
 			{
 				if (Lucene.Net.Store.FSDirectory.DISABLE_LOCKS)
 					return true;
+
+				// ADDED fhedberg@novell.com 16 Jun 2005
+				if (Enclosing_Instance.DisableLocks)
+					return true;
 				
 				bool tmpBool;
 				if (System.IO.File.Exists(Enclosing_Instance.lockDir.FullName))
@@ -111,6 +115,11 @@ namespace Lucene.Net.Store
 			{
 				if (Lucene.Net.Store.FSDirectory.DISABLE_LOCKS)
 					return ;
+				
+				// ADDED fhedberg@novell.com 16 Jun 2005 - disable locks per directory
+				if (Enclosing_Instance.DisableLocks)
+					return;
+				
 				bool tmpBool;
 				if (System.IO.File.Exists(lockFile.FullName))
 				{
@@ -135,6 +144,11 @@ namespace Lucene.Net.Store
 			{
 				if (Lucene.Net.Store.FSDirectory.DISABLE_LOCKS)
 					return false;
+
+				// ADDED fhedberg@novell.com 16 Jun 2005 - disable locks per directory
+				if (Enclosing_Instance.DisableLocks)
+					return false;
+
 				bool tmpBool;
 				if (System.IO.File.Exists(lockFile.FullName))
 					tmpBool = true;
@@ -184,6 +198,15 @@ namespace Lucene.Net.Store
                         }
                 }
 
+		// ADDED fhedberg@novell.com 16 Jun 2005 - disable locks per directory
+		
+		private bool disable_locks = false;
+
+		public bool DisableLocks {
+			get { return disable_locks; }
+			set { disable_locks = value; }
+		}
+
         private static System.Security.Cryptography.MD5 DIGESTER;
 		
 		/// <summary>A buffer optionally used in renameTo method </summary>
@@ -204,14 +227,22 @@ namespace Lucene.Net.Store
 		/// </returns>
 
 		/// CHANGED trow@novell.com 17 Mar 2005 set tmp dir at creation-time
+		/// CHANGED fhedberg@novell.com 16 Jun 2005 - disable locks per directory
 		public static FSDirectory GetDirectory(System.String path, System.String tmpDir, bool create)
 		{
-			return GetDirectory(new System.IO.FileInfo(path), tmpDir, create);
+			return GetDirectory(new System.IO.FileInfo(path), tmpDir, create, false);
 		}
 
+		// ADDED fhedberg@novell.com 16 Jun 2005 - disable locks per directory
+		public static FSDirectory GetDirectory(System.String path, System.String tmpDir, bool create, bool disable_locks)
+		{
+			return GetDirectory(new System.IO.FileInfo(path), tmpDir, create, disable_locks);
+		}
+
+		// CHANGED fhedberg@novell.com 16 Jun 2005 - disable locks per directory
 		public static FSDirectory GetDirectory(System.String path, bool create)
 		{
-			return GetDirectory(new System.IO.FileInfo(path), null, create);
+			return GetDirectory(new System.IO.FileInfo(path), null, create, false);
 		}
 		
 		/// <summary>Returns the directory instance for the named location.
@@ -227,7 +258,8 @@ namespace Lucene.Net.Store
 		/// </param>
 		/// <returns> the FSDirectory for the named file.  
 		/// </returns>
-		public static FSDirectory GetDirectory(System.IO.FileInfo file, System.String tmpDir, bool create)
+		/// CHANGED fhedberg@novell.com 16 Jun 2005 - disable locks per directory
+		public static FSDirectory GetDirectory(System.IO.FileInfo file, System.String tmpDir, bool create, bool disable_locks)
 		{
 			file = new System.IO.FileInfo(file.FullName);
 			FSDirectory dir;
@@ -236,7 +268,8 @@ namespace Lucene.Net.Store
 				dir = (FSDirectory) DIRECTORIES[file];
 				if (dir == null)
 				{
-					dir = new FSDirectory(file, tmpDir, create);
+					// CHANGED fhedberg@novell.com 16 Jun 2005 - disable locks per directory
+					dir = new FSDirectory(file, tmpDir, create, disable_locks);
 					DIRECTORIES[file] = dir;
 				}
 				else if (create)
@@ -251,18 +284,21 @@ namespace Lucene.Net.Store
 			return dir;
 		}
 
+		// CHANGED fhedberg@novell.com 16 Jun 2005 - disable locks per directory
 		public static FSDirectory GetDirectory(System.IO.FileInfo file, bool create)
 		{
-			return GetDirectory (file, null, create);
+			return GetDirectory (file, null, create, false);
 		}
 		
 		private System.IO.FileInfo directory = null;
 		private int refCount;
 		private System.IO.FileInfo lockDir;
 		
-		private FSDirectory(System.IO.FileInfo path, string tmpDir, bool create)
+		/// CHANGED fhedberg@novell.com 16 Jun 2005 - disable locks per directory
+		private FSDirectory(System.IO.FileInfo path, string tmpDir, bool create, bool disable_locks)
 		{
 			directory = path;
+			this.disable_locks = disable_locks;
 			
 			// FIXED joeshaw@novell.com  10 Jan 2005  Use TempDirectoryName to find where locks live
 			// FIXED trow@novell.com  17 Mar 2005  A fix on the fix
@@ -367,22 +403,25 @@ namespace Lucene.Net.Store
 		/// <summary>Returns the time the named file was last modified. </summary>
 		public override long FileModified(System.String name)
 		{
+			// FIXED joeshaw@novell.com 24 Jun 2005 - Use UTC
 			System.IO.FileInfo file = new System.IO.FileInfo(System.IO.Path.Combine(directory.FullName, name));
-			return ((file.LastWriteTime.Ticks - 621355968000000000) / 10000);
+			return ((file.LastWriteTimeUtc.Ticks - 621355968000000000) / 10000);
 		}
 		
 		/// <summary>Returns the time the named file was last modified. </summary>
 		public static long FileModified(System.IO.FileInfo directory, System.String name)
 		{
+			// FIXED joeshaw@novell.com 24 Jun 2005 - Use UTC
 			System.IO.FileInfo file = new System.IO.FileInfo(System.IO.Path.Combine(directory.FullName, name));
-			return ((file.LastWriteTime.Ticks - 621355968000000000) / 10000);
+			return ((file.LastWriteTimeUtc.Ticks - 621355968000000000) / 10000);
 		}
 		
 		/// <summary>Set the modified time of an existing file to now. </summary>
 		public override void  TouchFile(System.String name)
 		{
+			// FIXED joeshaw@novell.com 24 Jun 2005 - Use UTC
 			System.IO.FileInfo file = new System.IO.FileInfo(System.IO.Path.Combine(directory.FullName, name));
-            file.LastWriteTime = System.DateTime.Now;
+			file.LastWriteTimeUtc = System.DateTime.UtcNow;
 		}
 		
 		/// <summary>Returns the length in bytes of a file in the directory. </summary>
