@@ -57,6 +57,8 @@ namespace Beagle.Daemon {
 		public delegate double RelevancyMultiplier (Hit hit);
 
 		public event IIndexerChangedHandler ChangedEvent;
+		public event IIndexerChildIndexableHandler ChildIndexableEvent;
+		public event IIndexerUrisFilteredHandler UrisFilteredEvent;
 
 		/////////////////////////////////////////////////////
 		
@@ -87,11 +89,19 @@ namespace Beagle.Daemon {
 		private bool optimizing = false;
 		private int last_item_count = -1;
 
-		public LuceneDriver (string index_name) : this (index_name, 0) { }
+		/////////////////////////////////////////////////////
 
-		public LuceneDriver (string index_name, int index_version)
+		public LuceneDriver (string index_name) : this (index_name, 0, false) { }
+
+		public LuceneDriver (string index_name, string index_dir) : this (index_name, 0, false) { }
+
+		public LuceneDriver (string index_name, bool disable_locking) : this (index_name, 0, disable_locking) { }
+
+		public LuceneDriver (string index_name, int index_version) : this (index_name, index_version, false) { }
+
+		public LuceneDriver (string index_name, int index_version, bool disable_locking)
 		{
-			Setup (index_name, index_version);
+			Setup (index_name, index_version, disable_locking);
 		}
 
 		public string IndexDirectory {
@@ -118,9 +128,9 @@ namespace Beagle.Daemon {
 
 		/////////////////////////////////////////////////////
 
-		private void Setup (string index_name, int _minor_version)
+		private void Setup (string index_name, int _minor_version, bool disable_locking)
 		{			
-			top_dir = Path.Combine (PathFinder.StorageDir, index_name); 
+			top_dir = Path.Combine (PathFinder.StorageDir, index_name);
 			
 			string versionFile = Path.Combine (top_dir, "version");
 			string fingerprintFile = Path.Combine (top_dir, "fingerprint");
@@ -192,8 +202,12 @@ namespace Beagle.Daemon {
 				StreamReader sr = new StreamReader (fingerprintFile);
 				fingerprint = sr.ReadLine ();
 				sr.Close ();
-
 			} else {
+				// We can't create the index if we are in 
+				// read-only mode obviously.
+				if (disable_locking)
+					throw new InvalidOperationException ("LuceneDriver is in read-only mode, but requires index-creation");
+
 				// Purge and rebuild the index's directory
 				// structure.
 
@@ -222,10 +236,12 @@ namespace Beagle.Daemon {
 			}
 
 			Lucene.Net.Store.FSDirectory store;
-			store = Lucene.Net.Store.FSDirectory.GetDirectory (indexDir, lockDir, false);
+			
+			store = Lucene.Net.Store.FSDirectory.GetDirectory (indexDir, lockDir, false, disable_locking);
+			
 			ourStore = store;
 			ourStorePath = indexDir;
-
+			
 			//Store = store;
 
 			if (!indexExists) {
@@ -316,6 +332,7 @@ namespace Beagle.Daemon {
 
 			ArrayList added_uris;
 			ArrayList removed_uris;
+			ArrayList filtered_uris;
 			
 			lock (pending_by_uri) {
 				
@@ -326,6 +343,7 @@ namespace Beagle.Daemon {
 				pending_indexables = new ArrayList ();
 				added_uris = new ArrayList ();
 				removed_uris = new ArrayList ();
+				filtered_uris = new ArrayList ();
 
 				// Move our indexables and remove requests out of the
 				// hash and into local data structures.
@@ -361,9 +379,13 @@ namespace Beagle.Daemon {
 			IndexReader reader = IndexReader.Open (Store);
 			foreach (Uri uri in pending_uris) {
 				log.Debug ("- {0}", uri);
+
 				Term term = new Term ("Uri", uri.ToString ());
 				reader.Delete (term);
 				++removal_count;
+
+				term = new Term ("ParentUri", uri.ToString ());
+				removal_count += reader.Delete (term);
 			}
 			last_item_count = reader.NumDocs ();
 			reader.Close ();
@@ -375,12 +397,19 @@ namespace Beagle.Daemon {
 			// Step #2: Write out the pending adds
 			watch.Restart ();
 			IndexWriter writer = null;
+
+			// FIXME: Change this to a queue so that items 
+			// that are added while we flush (like children) also 
+			// gets committed.
+
 			foreach (Indexable indexable in pending_indexables) {
 				
 				Log.Debug ("+ {0}", indexable.DisplayUri);
 
+				Filter filter = null;
+
 				try {
-					FilterFactory.FilterIndexable (indexable);
+					FilterFactory.FilterIndexable (indexable, out filter);
 				} catch (Exception e) {
 					Log.Error ("Unable to filter {0} (mimetype={1})", indexable.DisplayUri, indexable.MimeType);
 					Log.Error (e);
@@ -402,6 +431,25 @@ namespace Beagle.Daemon {
 					++last_item_count;
 					++add_count;
 				}
+
+				if (filter != null) {
+					filtered_uris.Add (FilteredStatus.New (indexable, filter));
+				}
+
+				if (filter != null && filter.ChildIndexables.Count > 0) {
+					// Iterate across any indexables created by the
+					// filter and set up the parent-child relationship.
+					foreach (Indexable child in filter.ChildIndexables)
+						child.SetChildOf (indexable);
+					
+					// If nobody is listening for ChildIndexableEvent, 
+					// just add them ourselves so we wont loose them
+					if (ChildIndexableEvent != null)
+						ChildIndexableEvent ((Indexable[]) filter.ChildIndexables.ToArray (typeof (Indexable)));
+					else
+						foreach (Indexable child in filter.ChildIndexables) 
+							Add (child);
+				}
 			}
 			if (writer != null) 
 				writer.Close ();
@@ -415,6 +463,10 @@ namespace Beagle.Daemon {
 				ChangedEvent (this, added_uris, removed_uris, empty_collection);
 			}
 
+			if (filtered_uris.Count > 0 && UrisFilteredEvent != null) {
+				UrisFilteredEvent ((FilteredStatus[]) filtered_uris.ToArray (typeof (FilteredStatus)));
+			}
+
 			lock (pending_by_uri)
 				cycles_since_last_optimization++;
 
@@ -475,13 +527,13 @@ namespace Beagle.Daemon {
 			double t_lucene;
 			double t_assembly;
 
+			LNS.Query lucene_query = ToLuceneQuery (query, search_subset, bonus_uris);
+			if (lucene_query == null)
+				return;
+
 			Stopwatch sw = new Stopwatch ();
 			sw.Start ();
-			IndexReader reader = IndexReader.Open (Store);
-
-			LNS.Searcher searcher = new LNS.IndexSearcher (reader);
-			LNS.Query lucene_query = ToLuceneQuery (query, search_subset, bonus_uris);
-
+			LNS.Searcher searcher = new LNS.IndexSearcher (Store);
 			LNS.Hits hits = searcher.Search (lucene_query);
 			sw.Stop ();
 
@@ -520,6 +572,7 @@ namespace Beagle.Daemon {
 					double m = relevancy_multiplier (hit);
 					hit.ScoreMultiplier = (float) m;
 				}
+
 				result.Add (hit);
 			}
 
@@ -529,7 +582,6 @@ namespace Beagle.Daemon {
 
 			//////////////////////////////////////
 
-			// The call to searcher.Close () also closes the IndexReader.
 			searcher.Close ();
 
 			log.Debug ("{0}: n_hits={1} lucene={2:0.00}s assembly={3:0.00}s",
@@ -673,6 +725,11 @@ namespace Beagle.Daemon {
 			f = Field.Keyword ("Type", indexable.Type);
 			doc.Add (f);
 			
+			if (indexable.ParentUri != null) {
+				f = Field.Keyword ("ParentUri", UriFu.UriToSerializableString (indexable.ParentUri));
+				doc.Add (f);
+			}
+			
 			if (indexable.MimeType != null) {
 				f = Field.Keyword ("MimeType", indexable.MimeType);
 				doc.Add (f);
@@ -690,16 +747,19 @@ namespace Beagle.Daemon {
 				doc.Add (f);
 			}
 			
-			reader = indexable.GetTextReader ();
-			if (reader != null) {
-				f = Field.Text ("Text", reader);
-				doc.Add (f);
-			}
+			if (! indexable.NoContent) {
+				
+				reader = indexable.GetTextReader ();
+				if (reader != null) {
+					f = Field.Text ("Text", reader);
+					doc.Add (f);
+				}
 			
-			reader = indexable.GetHotTextReader ();
-			if (reader != null) {
-				f = Field.Text ("HotText", reader);
-				doc.Add (f);
+				reader = indexable.GetHotTextReader ();
+				if (reader != null) {
+					f = Field.Text ("HotText", reader);
+					doc.Add (f);
+				}
 			}
 
 			foreach (Property prop in indexable.Properties)
@@ -967,6 +1027,10 @@ namespace Beagle.Daemon {
 			if (str == null)
 				throw new Exception ("Got hit from Lucene w/o a Type!");
 			hit.Type = str;
+
+			str = doc.Get ("ParentUri");
+			if (str != null)
+				hit.ParentUri = UriFu.UriStringToUri (str);
 			
 			hit.MimeType = doc.Get ("MimeType");
 
@@ -1135,6 +1199,13 @@ namespace Beagle.Daemon {
 
 		/////////////////////////////////////////////////////
 
+		public static bool IsStopWord (string stemmed_word)
+		{
+			return ArrayFu.IndexOfString (StopAnalyzer.ENGLISH_STOP_WORDS, stemmed_word) != -1;
+		}
+
+		/////////////////////////////////////////////////////
+
 		//
 		// Helpful little utility functions
 		//
@@ -1151,6 +1222,27 @@ namespace Beagle.Daemon {
 
 		/////////////////////////////////////////////////////
 
+		public void Merge (string merge_dir)
+		{
+			string index_dir = Path.Combine (top_dir, "Index");
+			string locks_dir = Path.Combine (top_dir, "Locks");
+
+			if (!Directory.Exists (index_dir) || !Directory.Exists (locks_dir)) {
+				throw new Exception ("Index does not exists");
+			}
+			
+			// FIXME: Error recovery
+
+			Lucene.Net.Store.FSDirectory store = Lucene.Net.Store.FSDirectory.GetDirectory (index_dir, locks_dir, false);
+			Lucene.Net.Store.Directory[] stores = {store};
+			
+			IndexWriter writer = new IndexWriter (Store, null, false);
+			writer.AddIndexes (stores);
+			writer.Close ();
+		}
+
+		/////////////////////////////////////////////////////
+
 		// Expose some information for debugging and analytical purposes.
 
 		public void WriteIndexTermFrequencies (TextWriter writer)
@@ -1159,6 +1251,7 @@ namespace Beagle.Daemon {
 			TermEnum term_enum = reader.Terms ();
 
 			Term term;
+
 			while (term_enum.Next ()) {
 				term = term_enum.Term ();
 				int freq = term_enum.DocFreq ();
