@@ -64,9 +64,8 @@ namespace Beagle.Daemon {
 					// Send an end of message marker
 					this.client.GetStream ().WriteByte (0xff);
 					this.client.Flush ();
-				} catch (IOException e) {
-					// The socket was shut down, so we can't write
-					// any more.
+				} catch (Exception e) {
+					Logger.Log.Debug ("Caught an exception sending response; socket shut down: {0}", e.Message);
 					return false;
 				}
 
@@ -108,7 +107,12 @@ namespace Beagle.Daemon {
 
 		public void WatchCallback (IAsyncResult ar)
 		{
-			int bytes_read = this.client.GetStream ().EndRead (ar);
+			int bytes_read = 0;
+
+			try {
+				bytes_read = this.client.GetStream ().EndRead (ar);
+			} catch (SocketException) {
+			} catch (IOException) { }
 
 			if (bytes_read == 0)
 				Close ();
@@ -118,6 +122,11 @@ namespace Beagle.Daemon {
 
 		private void SetupWatch ()
 		{
+			if (this.client == null) {
+				this.Close ();
+				return;
+			}
+
 			this.client.GetStream ().BeginRead (new byte[1024], 0, 1024,
 							    new AsyncCallback (WatchCallback), null);
 		}
@@ -177,7 +186,7 @@ namespace Beagle.Daemon {
 
 				if (bytes_read > 0) {
 					// 0xff signifies end of message
-					end_index = Array.IndexOf (network_data, (byte) 0xff);
+					end_index = ArrayFu.IndexOfByte (network_data, (byte) 0xff);
 
 					buffer_stream.Write (network_data, 0,
 							     end_index == -1 ? bytes_read : end_index);
