@@ -28,6 +28,7 @@
 using System;
 using System.Collections;
 using System.IO;
+using System.Threading;
 
 using Mono.Data.SqliteClient;
 
@@ -45,29 +46,29 @@ namespace Beagle.Daemon {
 		static TextCache () 
 		{
 			text_cache_dir = Path.Combine (PathFinder.StorageDir, "TextCache");
-			if (! Directory.Exists (text_cache_dir))
+			if (! Directory.Exists (text_cache_dir)) {
 				Directory.CreateDirectory (text_cache_dir);
 
-			// Create our cache subdirectories.
-			for (int i = 0; i < 256; ++i) {
-				string subdir = i.ToString ("x");
-				if (i < 16)
-					subdir = "0" + subdir;
-				subdir = Path.Combine (text_cache_dir, subdir);
-				if (! Directory.Exists (subdir))
+				// Create our cache subdirectories.
+				for (int i = 0; i < 256; ++i) {
+					string subdir = i.ToString ("x");
+					if (i < 16)
+						subdir = "0" + subdir;
+					subdir = Path.Combine (text_cache_dir, subdir);
 					Directory.CreateDirectory (subdir);
+				}
 			}
-
+			
 			// Create our Sqlite database
 			string db_filename = Path.Combine (text_cache_dir, "TextCache.db");
 			bool create_new_db = false;
 			if (! File.Exists (db_filename))
 				create_new_db = true;
-
+			
 			connection = new SqliteConnection ();
 			connection.ConnectionString = "URI=file:" + db_filename;
 			connection.Open ();
-
+			
 			if (create_new_db) {
 				DoNonQuery ("CREATE TABLE uri_index (            " +
 					    "  uri      STRING UNIQUE NOT NULL,  " +
@@ -93,10 +94,51 @@ namespace Beagle.Daemon {
 		private static void DoNonQuery (string format, params object [] args)
 		{
 			SqliteCommand command = NewCommand (format, args);
-			command.ExecuteNonQuery ();
+			while (true) {
+				try {
+					command.ExecuteNonQuery ();
+					break;
+				} catch (SqliteException ex) {
+					// FIXME: should we eventually time out?
+					if (ex.SqliteError == SqliteError.BUSY)
+						Thread.Sleep (50);
+					else
+						throw ex;
+				}
+			}
 			command.Dispose ();
 		}
 
+		static private SqliteDataReader ExecuteReaderOrWait (SqliteCommand command)
+		{
+			SqliteDataReader reader = null;
+			while (reader == null) {
+				try {
+					reader = command.ExecuteReader ();
+				} catch (SqliteException ex) {
+					if (ex.SqliteError == SqliteError.BUSY)
+						Thread.Sleep (50);
+					else
+						throw ex;
+				}
+			}
+			return reader;
+		}
+
+		static private bool ReadOrWait (SqliteDataReader reader)
+		{
+			while (true) {
+				try {
+					return reader.Read ();
+				} catch (SqliteException ex) {
+					if (ex.SqliteError == SqliteError.BUSY)
+						Thread.Sleep (50);
+					else
+						throw ex;
+				}
+			}
+		}
+
 		private static void Insert (Uri uri, string filename)
 		{
 			DoNonQuery ("INSERT OR REPLACE INTO uri_index (uri, filename) VALUES ('{0}', '{1}')",
@@ -106,14 +148,14 @@ namespace Beagle.Daemon {
 		private static string LookupPathRawUnlocked (Uri uri, bool create_if_not_found)
 		{
 			SqliteCommand command;
-			SqliteDataReader reader;
+			SqliteDataReader reader = null;
 			string path = null;
 
 			command = NewCommand ("SELECT filename FROM uri_index WHERE uri='{0}'", 
 			                      UriToString (uri));
-			reader = command.ExecuteReader ();
-			if (reader.Read ())
-				path = reader [0].ToString ();
+			reader = ExecuteReaderOrWait (command);
+			if (ReadOrWait (reader))
+				path = reader.GetString (0);
 			reader.Close ();
 			command.Dispose ();
 
@@ -161,6 +203,9 @@ namespace Beagle.Daemon {
 			FileStream stream;
 			stream = new FileStream (path, FileMode.Create, FileAccess.Write, FileShare.ReadWrite);
 
+			// We don't expect to need this again in the near future.
+			FileAdvise.FlushCache (stream);
+
 			StreamWriter writer;
 			writer = new StreamWriter (stream);
 			return writer;
