@@ -48,44 +48,49 @@ namespace Beagle.WebService {
 		public static string hostname = "localhost";		
 		public static string DEFAULT_XSP_ROOT = Path.Combine (ExternalStringsHack.PkgDataDir, "xsp");
 		public static string DEFAULT_XSP_PORT = "8888";
-
-		public static bool web_global = false;
-		public static bool web_start = false;
-		public static string web_port = DEFAULT_XSP_PORT;
 		public static string web_rootDir = DEFAULT_XSP_ROOT;
+		public static string web_port = DEFAULT_XSP_PORT;		
+		public static bool web_start = true;
+		public static bool web_global;
 
-		public static ExternalAccessFilter AccessFilter;																
 		static Mono.ASPNET.ApplicationServer appServer = null;
-		static string DEFAULT_APP_MAPPINGS = "/:" + DEFAULT_XSP_ROOT + ",/beagle:" + DEFAULT_XSP_ROOT;
-
 		//Both "/" and "/beagle" aliased to DEFAULT_XSP_ROOT only for BeagleXSP server
-		static string[] xsp_param = {"--port", DEFAULT_XSP_PORT,
-					     "--root", DEFAULT_XSP_ROOT, 
-					     "--applications", DEFAULT_APP_MAPPINGS, 
-					     "--nonstop"};
+		static string DEFAULT_APP_MAPPINGS = "/:" + DEFAULT_XSP_ROOT + ",/beagle:" + DEFAULT_XSP_ROOT;
+		static string[] xsp_param = {"--port", 	DEFAULT_XSP_PORT,
+					     			 "--root", 	DEFAULT_XSP_ROOT, 
+					     			 "--applications", DEFAULT_APP_MAPPINGS, 
+					     			 "--nonstop"};
+					     			 
+		//static Logger log = Logger.Get ("WebServiceBackEnd");		
+		static string 		BeagleHttpUriBase;
+		static string[] 	reserved_suffixes;
+
+		public static ExternalAccessFilter AccessFilter;
 		
 		public static void Start()
 		{			
 			try {
 				hostname = Dns.GetHostName();
-				Console.WriteLine("This Computer Hostname: " + hostname);
+				Logger.Log.Info("This Computer Hostname: " + hostname);
 			}
 			catch (Exception ex) 
 			{
-				Console.WriteLine("Caught exception {0} in Dns.GetHostName: ", ex.Message);
-				Console.WriteLine("Resetting hostname to \"localhost\"");
+				Logger.Log.Error("Caught exception {0} in Dns.GetHostName: ", ex.Message);
+				Logger.Log.Error("Resetting hostname to \"localhost\"");
 				hostname = "localhost";
 			}
-								
+			
+			web_global = Conf.WebServices.AllowGlobalAccess;
+			
 			//start web-access server first
 			Logger.Log.Debug ("Starting WebBackEnd");
-			WebBackEnd.init (web_global);
+			WebBackEnd.init ();
 
 			//Next start web-service server
 			Logger.Log.Info ("Starting WebServiceBackEnd");
-			WebServiceBackEnd.init (web_global);
+			WebServiceBackEnd.init ();
 
-			Logger.Log.Debug ("Global WebAccess {0}", web_global ? "Enabled" : "Disabled");
+			Logger.Log.Debug ("Global WebServicesAccess {0}", web_global ? "Enabled" : "Disabled");
 
 			xsp_param[1] = web_port;
 			xsp_param[3] = web_rootDir;
@@ -96,22 +101,22 @@ namespace Beagle.WebService {
 				xsp_param[5] = "/:" + web_rootDir + ",/beagle:" + web_rootDir + "/beagle";
 			
 			try {
-				//",/beagle/local:" + ExternalStringsHack.Prefix,	
+					// Mapping /beagle/local to ExternalStringsHack.Prefix	
 				if (Directory.Exists(ExternalStringsHack.Prefix))
 				xsp_param[5] += ",/beagle/local:" + ExternalStringsHack.Prefix;
 							
-				//",/beagle/gnome:" + ExternalStringsHack.GnomePrefix +
+					//Mapping /beagle/gnome to ExternalStringsHack.GnomePrefix
 				if (Directory.Exists(ExternalStringsHack.GnomePrefix))
 				xsp_param[5] += ",/beagle/gnome:" + ExternalStringsHack.GnomePrefix;
 												
-				//",/beagle/kde3:" + ExternalStringsHack.KdePrefix +
+					//Mapping /beagle/kde3 to ExternalStringsHack.KdePrefix
 				if (Directory.Exists(ExternalStringsHack.KdePrefix))
 				xsp_param[5] += ",/beagle/kde3:" + ExternalStringsHack.KdePrefix;
 				
 				//if (!hostname.Equals("localhost")) {
 
-					string[] reserved_suffixes = new string[] {"local", "gnome", "kde3" };
-					string BeagleHttpUriBase = "http://" + hostname + ":" + xsp_param[1] + "/beagle/";
+					reserved_suffixes = new string[] {"beagle", "local", "gnome", "kde3"};
+					BeagleHttpUriBase = "http://" + hostname + ":" + xsp_param[1] + "/beagle/";
 				
 					AccessFilter = new ExternalAccessFilter(BeagleHttpUriBase, reserved_suffixes);
 				
@@ -148,7 +153,7 @@ namespace Beagle.WebService {
 				}
 				else
 					Logger.Log.Debug("BeagleXSP Applications list: " + xsp_param[5]);
-			}				
+			}
 		}
 		
 		public static void Stop() 
@@ -159,21 +164,67 @@ namespace Beagle.WebService {
 				appServer = null;
 			}
 		}
-
-/////////////////////////////////////////////////////////////////////////////////////////
 		
+/////////////////////////////////////////////////////////////////////////////////////////
+
+		private void WebServicesConfigurationChanged (Conf.Section section)
+		{			
+			Logger.Log.Info("WebServicesConfigurationChanged EventHandler invoked");
+			if (! (section is Conf.WebServicesConfig))
+				return;
+			
+			Conf.WebServicesConfig wsc = (Conf.WebServicesConfig) section;
+			
+			if (web_global != wsc.AllowGlobalAccess) {
+				// Update AllowGlobalAccess configuration:
+				web_global = wsc.AllowGlobalAccess;
+				Logger.Log.Info("WebServicesBackEnd: Global WebServicesAccess {0}", web_global ? "Enabled" : "Disabled");
+			}
+			
+			if (wsc.PublicFolders.Count == 0)
+				return;
+				
+			Logger.Log.Warn("WebServicesBackEnd: Changes in PublicFolders configuration doesn't take effect until Beagle daemon is restarted!");
+				
+/*			Note: ExternalAccessFilter Matchers can be updated, 
+				  but app mapping changes in BeagleXsp Server require it to be restarted !
+			
+			ArrayList newList = new ArrayList();
+			foreach (string pf in wsc.PublicFolders) {
+					if (pf == null || pf == "")
+						continue;					
+	
+					newList.Add (new NetBeagleHandler (host, port, this));								
+			}	
+			
+			if (usingPublicFoldersDotCfgFile) {
+				usingPublicFoldersDotCfgFile = false;
+				log.Warn("NetBeagleQueryable: Duplicate configuration of PublicFolders in '~/.beagle/publicfolders.cfg' and '~/.beagle/config/webservices.xml' !");
+				log.Info("NetBeagleQueryable: Remove '~/.beagle/publicfolders.cfg' file. Use 'beagle-config' instead to setup public folders.");		
+				log.Info("NetBeagleQueryable: Replacing PublicFoldersList with new list from \"webservices.xml\" having {0} node(s)", newList.Count);
+			}			
+			
+			AccessFilter.ReplaceAccessFilter(newList);  			
+*/					 
+		}
+		
+/////////////////////////////////////////////////////////////////////////////////////////		
+				
 		//KNV: If needed, we can convert this to a Singleton, adding a 
 		//	   static Factory method to get the singleton instance reference,
 		//	   so that front-end code always gets hold of same instance.
-		static WebServiceBackEnd instance = null;
-		static bool allow_global_access = false;
 		
+		static WebServiceBackEnd instance = null;		
+
 		private Hashtable resultTable;
 		private Hashtable sessionTable;
 		
 		public WebServiceBackEnd() {
+
 			 resultTable 		= Hashtable.Synchronized(new Hashtable());
 			 sessionTable 		= Hashtable.Synchronized(new Hashtable());
+			
+			Conf.Subscribe (typeof (Conf.WebServicesConfig), new Conf.ConfigUpdateHandler (WebServicesConfigurationChanged));							 
 		}
 
 		~WebServiceBackEnd() {
@@ -182,13 +233,11 @@ namespace Beagle.WebService {
 		}
 	
 		public bool allowGlobalAccess {
-			get { return allow_global_access; }
+			get { return web_global; }		
 		}
 				
-		public static void init(bool web_global)
+		public static void init()
 		{
-		    allow_global_access = web_global;
-			
 		    if (instance == null) {
 
 		  		instance = new WebServiceBackEnd();
@@ -200,7 +249,7 @@ namespace Beagle.WebService {
 				 	"WebServiceBackEnd.rem", WellKnownObjectMode.Singleton);
 		  		RemotingConfiguration.ApplicationName="beagled";
 		  		RemotingConfiguration.RegisterWellKnownServiceType(WKSTE);
-		    }	    
+		    }	 
 		}
 
 		void OnHitsAdded (QueryResult qres, ICollection hits)
@@ -215,15 +264,14 @@ namespace Beagle.WebService {
 					lock (results.SyncRoot) 
 						results.AddRange(hits);
 				}
-				else {
-				
-					//Query query = sdata.query;					
+				else {				
+						//Query query = sdata.query;					
 					lock (results.SyncRoot) {
-						foreach (Hit h in hits)
-							if (AccessFilter.FilterHit(h))
+						foreach (Hit h in hits) 
+							if (h.Uri.ToString().StartsWith(NetworkedBeagle.BeagleNetPrefix) ||
+											AccessFilter.FilterHit(h))
 								results.Add(h);
 					}
-					//Console.WriteLine("OnHitsAdded: Total hits in Results is {0}", results.Count); 												
 				}
 			}
 		}
@@ -251,7 +299,6 @@ namespace Beagle.WebService {
 
 		void OnFinished (QueryResult qres)
 		{
-
 			DetachQueryResult (qres);
 		}
 
@@ -336,13 +383,13 @@ namespace Beagle.WebService {
 		public const int SC_INVALID_QUERY = -1;
 		public const int SC_UNAUTHORIZED_ACCESS = -2;
 		public const int SC_INVALID_SEARCH_TOKEN = -3;
-
+		public const int SC_DUPLICATE_QUERY = -4;
+		
 		//Full beagledQuery
 		public SearchResult doQuery(SearchRequest sreq, bool isLocalReq)
 		{	
-			SearchResult sr;
-			//if (sreq == (MarshalByRef)(null))
-				//return new SearchResult();
+			SearchResult sr = null;
+
 			if (sreq.text == null || sreq.text.Length == 0 ||
 				(sreq.text.Length == 1 && sreq.text[0].Trim() == "") ) {
 				
@@ -351,13 +398,13 @@ namespace Beagle.WebService {
 			    sr.statusMsg = "Error: No search terms specified";
 				return sr;
 			}
-				
+
 			Query query = new Query();
-			
+						
 			foreach (string text in sreq.text) 
 				query.AddText(text);				
 			
-			Console.WriteLine("WebServiceBackEnd: Received {0} WebService Query with search term: {1}", isLocalReq ? "Local":"External", query.QuotedText);
+			Logger.Log.Info("WebServiceBackEnd: Received {0} WebService Query with search term: {1}", isLocalReq ? "Local":"External", query.QuotedText);
 
 			if (sreq.mimeType != null && sreq.mimeType[0] != null)
 				foreach (string mtype in sreq.mimeType)
@@ -367,30 +414,60 @@ namespace Beagle.WebService {
 				foreach (string src in sreq.searchSources)
 					query.AddSource(src);
 
-			//FIXME: Add check to restrict queryDomain to Local or Neighborhood ?
-			//Having this Global can cause cascading/looping of requests.
+			//If needed, check to restrict queries to System or Neighborhood domain, can be added here
 			if (sreq.qdomain > 0)
 				query.AddDomain(sreq.qdomain);
+						
+			if (!isLocalReq) {	//External Request, check if this Node is already processing it
+
+			 	lock (this) {					
+					if ((sreq.searchId != 0) && NetworkedBeagle.IsCachedRequest(sreq.searchId)) {
+
+						sr = new SearchResult();
+				    	sr.numResults = sr.totalResults = sr.firstResultIndex = 0;
+						sr.hitResults = new HitResult[sr.numResults];	
+				 		sr.searchToken = "";
+
+				 		sr.statusCode = SC_DUPLICATE_QUERY;
+				 		sr.statusMsg = "Error: Duplicate Query loopback";
+				 		Logger.Log.Warn("WebServiceBackEnd: Received duplicate Query for a query already in process!");
+				 		Logger.Log.Warn("WebServiceBackEnd: Check NetBeagle configuration on all nodes to remove possible loops");
+				 	}
+	
+					if (sreq.hopCount >= 5)  {
+						//If request has traversed 5 nodes in reaching here, stop cascading. 
+						//Make it a Local Query.
+						query.RemoveDomain(sreq.qdomain);
+						query.AddDomain(QueryDomain.Local);
+				 	}
+				 					 	
+					if ((sr == null) && (sreq.searchId != 0) )
+						NetworkedBeagle.CacheRequest(query, sreq.searchId, sreq.hopCount + 1);				 	
+				 }
+				 
+				 if (sr != null)
+				 	return sr;	
+				 	
+				 //Logger.Log.Info("New external Query: searchId = {0}", sreq.searchId); 	
+			}
 
 			ArrayList results = ArrayList.Synchronized(new ArrayList());
 			
 			QueryResult qres = new QueryResult ();
-			
-			//Console.WriteLine("WebServiceBackEnd: Starting Query for string \"{0}\"",	query.QuotedText);
 
-			string searchId = TokenGenerator();
+			string searchToken = TokenGenerator();
 						
 			SessionData sdata = new SessionData(query, results, isLocalReq);
 				
 			AttachQueryResult (qres, sdata);
 			
 /* Include this code, if sessionID passed from front-end:
-			if (sessionTable.Contains(searchId))
-				sessionTable[searchId] = sdata;
+			if (sessionTable.Contains(searchToken))
+				sessionTable[searchToken] = sdata;
 			else
 */
-			sessionTable.Add(searchId, sdata);
-		
+			sessionTable.Add(searchToken, sdata);
+			
 			QueryDriver.DoQuery (query, qres);
 
 			while (resultTable.Contains(qres) && (results.Count < MAX_RESULTS_PER_CALL) )
@@ -405,7 +482,8 @@ namespace Beagle.WebService {
 			
 				sr.numResults = results.Count < MAX_RESULTS_PER_CALL ? results.Count: MAX_RESULTS_PER_CALL;	
 				sr.hitResults = new HitResult[sr.numResults];
-			    				
+			    
+			    string hitUri;			
 				for (int i = 0; i < sr.numResults; i++) {
 				
 					Hit h = (Hit) results[i];
@@ -416,20 +494,17 @@ namespace Beagle.WebService {
 					if (queryable == null)
 						snippet = "ERROR: hit.SourceObject is null, uri=" + h.Uri;
 					else
-						snippet = queryable.GetSnippet (ICollection2StringList(query.Text), h);				
-					
-					//snippet == "", implies GetSnippet returned empty snippet
-					if (snippet == null)   	
-						snippet = "";		
+						snippet = queryable.GetSnippet (ICollection2StringList(query.Text), h);											
 								
 					sr.hitResults[i] = new HitResult();
 					sr.hitResults[i].id = h.Id;
 					
-					if (isLocalReq)
-						sr.hitResults[i].uri = h.Uri.ToString();
-					else 
-						sr.hitResults[i].uri = AccessFilter.TranslateHit(h);
-
+					hitUri = h.Uri.ToString();
+					if (isLocalReq || hitUri.StartsWith(NetworkedBeagle.BeagleNetPrefix))
+							sr.hitResults[i].uri = hitUri;
+					else
+							sr.hitResults[i].uri = AccessFilter.TranslateHit(h);
+					
 	        	    sr.hitResults[i].resourceType = h.Type;
 					sr.hitResults[i].mimeType = h.MimeType;
 					sr.hitResults[i].source = h.Source;
@@ -447,7 +522,8 @@ namespace Beagle.WebService {
 						sr.hitResults[i].properties[j].IsSearched = p.IsSearched;							
 					}
 
-					sr.hitResults[i].snippet = snippet;
+					if (snippet != null)
+						sr.hitResults[i].snippet = snippet.Trim();
 				}					
 			   } //end lock
 			 }// end if
@@ -463,11 +539,11 @@ namespace Beagle.WebService {
 			 sr.searchToken = "";
 				
 			 if (sr.totalResults > 0)
-				sr.searchToken = searchId;
+				sr.searchToken = searchToken;
 					
 			 sr.statusCode = SC_QUERY_SUCCESS;
 			 sr.statusMsg = "Success";
-			 Console.WriteLine("WebServiceBackEnd: Total Results = "  + sr.totalResults);			
+			 Logger.Log.Info("WebServiceBackEnd: Total Results = "  + sr.totalResults);			
 			 return sr;
 		}
 
@@ -480,7 +556,7 @@ namespace Beagle.WebService {
 			if (!sessionTable.ContainsKey(searchToken)) {
 				sr.statusCode = SC_INVALID_SEARCH_TOKEN;
 				sr.statusMsg = "Error: Invalid Search Token";
-				Console.WriteLine("GetMoreResults: Invalid Search Token received ");
+				Logger.Log.Warn("GetMoreResults: Invalid Search Token received ");
 				return sr;
 			}
 									
@@ -488,11 +564,11 @@ namespace Beagle.WebService {
 			if (results == null) {
 				sr.statusCode = SC_INVALID_SEARCH_TOKEN;
 				sr.statusMsg = "Error: Invalid Search Token";
-				Console.WriteLine("GetMoreResults: Invalid Search Token received ");
+				Logger.Log.Warn("GetMoreResults: Invalid Search Token received ");
 				return sr;
 			}
 
-			lock (results.SyncRoot) { //Lock results ArrayList to prevent more Hits added till we've processed doQuery
+			lock (results.SyncRoot) { //Lock results ArrayList to prevent more Hits getting added till we've processed doQuery
  
  				int i = 0;
  				
@@ -501,31 +577,23 @@ namespace Beagle.WebService {
 				
 				sr.hitResults = new HitResult[sr.numResults];
 			
+				string hitUri;
 				for (int k = startIndex; (i < sr.numResults) && (k < results.Count); k++)   {		
 				
 					Hit h = (Hit) results[k];	
 							
 					sr.hitResults[i] = new HitResult();
 					
-/* 	 GetMoreResults will NOT return Snippets by default. Client must make explicit GetSnippets request to get snippets for these hits.
-
-					string snippet = ""; 						
-					Queryable queryable = h.SourceObject as Queryable;
-					if (queryable == null)
-						snippet = "ERROR: hit.SourceObject is null, uri=" + h.Uri;
-					else
-						snippet = queryable.GetSnippet (ICollection2StringList(query.Text), h);		
-					 = snippet;			
-*/
-// Not initializing sr.hitResults[i].snippet implies there is no <snippets> element in HitResult XML response
-// which implies GetSnippet was not done.
+// GetMoreResults will NOT return Snippets by default. Client must make explicit GetSnippets request to get snippets for these hits.
+// Not initializing sr.hitResults[i].snippet implies there is no <snippets> element in HitResult XML response.
 								
 					sr.hitResults[i].id = h.Id;
-					
-					if (isLocalReq)
-						sr.hitResults[i].uri = h.Uri.ToString();
-					else 
-						sr.hitResults[i].uri = AccessFilter.TranslateHit(h);
+							
+					hitUri = h.Uri.ToString();
+					if (isLocalReq || hitUri.StartsWith(NetworkedBeagle.BeagleNetPrefix))
+							sr.hitResults[i].uri = hitUri;
+					else
+							sr.hitResults[i].uri = AccessFilter.TranslateHit(h);
 
 	        	    sr.hitResults[i].resourceType = h.Type;
 					sr.hitResults[i].mimeType = h.MimeType;
@@ -562,7 +630,7 @@ namespace Beagle.WebService {
 			return sr;
 		}
 		
-		static string InvalidHitSnippetError = "ERROR: Invalid or Duplicate Hit Id";
+		public static string InvalidHitSnippetError = "ERROR: Invalid or Duplicate Hit Id";
 		public HitSnippet[] getSnippets(string searchToken, int[] hitIds)
 		{	
 			HitSnippet[] response;
@@ -570,7 +638,7 @@ namespace Beagle.WebService {
 			if (!sessionTable.ContainsKey(searchToken)) {
 			
 				response = new HitSnippet[0];
-				Console.WriteLine("GetSnippets: Invalid Search Token received ");
+				Logger.Log.Warn("GetSnippets: Invalid Search Token received ");
 				return response;
 			}
 									
@@ -578,14 +646,15 @@ namespace Beagle.WebService {
 			if ((results == null) || (results.Count == 0)) {
 
 				response = new HitSnippet[0];
-				Console.WriteLine("GetSnippets: Invalid Search Token received ");
+				Logger.Log.Warn("GetSnippets: Invalid Search Token received ");
 				return response;
 			}
-
+			
 			int i = 0; 		
 			ArrayList IdList = new ArrayList();
 			IdList.AddRange(hitIds);	
 			response = new HitSnippet[hitIds.Length];
+			Logger.Log.Debug("GetSnippets invoked with {0} hitIds", hitIds.Length);
 
 			Query query = ((SessionData)sessionTable[searchToken]).query;
 			
@@ -603,8 +672,15 @@ namespace Beagle.WebService {
 								snippet = "ERROR: hit.SourceObject is null, uri=" + h.Uri;
 							else
 								snippet = queryable.GetSnippet (ICollection2StringList(query.Text), h);		
-										
-							response[i++] = new HitSnippet(h.Id, snippet);		
+							
+							//GetSnippets always invoked on Target Beagle Node where hits originate:
+							if (snippet == null)
+								snippet = "";
+								
+							HitSnippet hs = new HitSnippet();
+							hs.hitId = h.Id; 
+							hs.snippet = snippet.Trim();
+							response[i++] = hs;		
 					
 							if (i == hitIds.Length)
 								return response;
@@ -613,11 +689,16 @@ namespace Beagle.WebService {
 			} //end lock
 			
 			foreach (int hitId in IdList) {
-					response[i++] = new HitSnippet(hitId, InvalidHitSnippetError);
+					HitSnippet hs = new HitSnippet();
+					hs.hitId = hitId; 
+					hs.snippet = InvalidHitSnippetError;
+					response[i++] = hs;	
+
 					if (i == hitIds.Length)
 							break;
 			}		
-
+			Logger.Log.Warn("GetSnippets invoked some invalid hitIds");
+			
 			return response;
 		}
 		
@@ -646,6 +727,9 @@ namespace Beagle.WebService {
 		public string[] mimeType;
 		public string[] searchSources;
 		public QueryDomain qdomain;
+		//Unique searchId across network
+		public int 	searchId;			
+		public int 	hopCount;
 	}
 
 	[Serializable()]
@@ -709,13 +793,6 @@ namespace Beagle.WebService {
 	public class HitSnippet {
 		public int hitId;
 		public string snippet;
-		
-		public HitSnippet() { hitId = 0; snippet = null; }
-		public HitSnippet( int i, string s) {
-			this.hitId = i;
-			this.snippet =s;
-		}
-
 	}
 		
 }
